@@ -8,8 +8,10 @@ import git
 import gitlab
 from enum import Enum
 import csv
+from glob import glob
 
 from result import Ok, Err, Result
+from collections.abc import Mapping
 
 KEY_STUDENT_ID = 'id'
 KEY_STUDENT_NAME_FIRST = 'first_name'
@@ -42,7 +44,7 @@ def students_csv_fieldnames():
 
 
 def teams_csv_fieldnames():
-    return [KEY_TEAM_MEMBER_1, KEY_TEAM_MEMBER_2, KEY_TEAM_MEMBER_3, KEY_TEAM_NAME, KEY_TEAM_PATH]
+    return ['member', 'team']
 
 
 def escape_csv_string(string):
@@ -52,7 +54,7 @@ def escape_csv_string(string):
 
 
 def validate_headers(headers):
-    return all({k == headers[k]: headers[k] for k in headers})
+    return headers != None and all({k.strip() == headers[k].strip(): headers[k] for k in headers})
 
 
 def check_row(row):
@@ -310,7 +312,7 @@ class Ep2:
         try:
             return self.config.get('Personal', 'Gender')
         except configparser.NoSectionError:
-            return 'female'
+            return 'diverse'
 
     def template_path(self, template):
         """Returns the path of the template file to be used"""
@@ -326,30 +328,50 @@ class Ep2:
         # TODO add caching
         return Ep2Group(self, group)
 
-    def idea_checkout(self, project):
+    def idea_checkout(self, project, group: str = None, ue: int = None, copy_pre_eval: bool = False, copy_tests: bool = False):
         src = self.local_repo(project)
         dst = self.config.get('Local', 'IdeaEvalDir')
 
+        dst_src_dir = os.path.join(dst, 'src')
+        dst_angabe_dir = os.path.join(dst, 'angabe')
+        dst_test_dir = os.path.join(dst, 'test')
+
+        src_src_dir = os.path.join(src, 'src')
+        src_angabe_dir = os.path.join(src, 'angabe')
+
         if os.path.exists(dst):
-            for the_file in os.listdir(dst):
-                file_path = os.path.join(dst, the_file)
-                try:
-                    if os.path.isfile(file_path):
-                        os.unlink(file_path)
-                    elif os.path.isdir(file_path) and the_file != '.idea':
-                        shutil.rmtree(file_path)
-                except Exception as e:
-                    print(e)
+
+            if os.path.exists(dst_src_dir):
+                os.unlink(dst_src_dir)
+
+            if os.path.exists(dst_angabe_dir):
+                os.unlink(dst_angabe_dir)
+
+            if os.path.exists(dst_test_dir):
+                os.unlink(dst_test_dir)
+
+            pre_eval_results = glob(os.path.join(dst, 'pre_eval_*.yml'))
+
+            for pre_eval_result in pre_eval_results:
+                os.unlink(pre_eval_result)
+
         else:
             os.makedirs(dst, exist_ok=True)
 
-        for item in os.listdir(src):
-            s = os.path.join(src, item)
-            d = os.path.join(dst, item)
-            if os.path.isdir(s) and item != '.git' and item != '.idea':
-                shutil.copytree(s, d)
-            elif os.path.isfile(s):
-                shutil.copy2(s, d)
+        os.symlink(src_src_dir, dst_src_dir, target_is_directory=True)
+        os.symlink(src_angabe_dir, dst_angabe_dir, target_is_directory=True)
+
+        if copy_pre_eval and ue is not None:
+            src_pre_eval = os.path.join(self.tutor_repo(), group, project, 'pre_eval_{}.yml'.format(ue))
+            dst_pre_eval = os.path.join(dst, 'pre_eval_{}.yml'.format(ue))
+
+            os.symlink(src_pre_eval, dst_pre_eval)
+
+        if copy_tests and ue is not None:
+            src_test_dir = os.path.join(self.tutor_repo(), 'unittests', str(ue))
+
+            if os.path.exists(src_test_dir):
+                os.symlink(src_test_dir, dst_test_dir, target_is_directory=True)
 
     def test_info_csv(self) -> str:
         return os.path.join(self.tutor_repo(), "test_info.csv")
@@ -437,7 +459,7 @@ def verify_and_normalize_student_id(student_id) -> Result:
     if re.match('\\d*', student_id) and len(student_id) <= 8:
         return Ok('0' * (8 - len(student_id)) + student_id)
     else:
-        return Err('invalid student id')
+        return Err('invalid student id: {}'.format(student_id))
 
 
 def validate_student_id(ctx, param, value):
@@ -596,3 +618,68 @@ class FileInformation:
             del self.files[file_path]
         else:
             self.add_file(file_path, Action.DELETE, push)
+
+
+class Ep2Team:
+    students: [str]
+    name: str
+
+    def __init__(self, name: str):
+        self.name = name
+        self.students = []
+
+
+class TeamIndex:
+    marked: [str]
+    team_index: {Ep2Team}
+    teams: {Ep2Team}
+    remaining_teams: [Ep2Team]
+
+    def __init__(self, group: Ep2Group):
+        self.students = group.student_list()
+        csv_file = group.team_csv()
+        self.marked = []
+        self.team_index = {}
+        self.teams = {}
+        with open(csv_file, 'r') as infile:
+            reader = csv.DictReader(infile, teams_csv_fieldnames(), KEY_INVALID, strict=True)
+
+            headers = next(reader, None)
+            if not validate_headers(headers):
+                click.secho('Malformed file: %s. Invalid headers!' % csv_file)
+                exit(1)
+
+            for row in reader:
+                member = row['member']
+                team_name = row['team']
+
+                if member not in self.students:
+                    click.secho('[WARN] potential data inconsistency detected: student {} in team list but not in '
+                                'student list!'.format(member), fg='yellow', bold=True)
+
+                if team_name in self.teams:
+                    team = self.teams[team_name]
+                else:
+                    team = Ep2Team(team_name)
+                    self.teams[team_name] = team
+
+                team.students += [member]
+                self.team_index[member] = team
+
+        self.remaining_teams = self.teams.values()
+
+    def other_students(self, student: str) -> [str]:
+        return list(filter(lambda x: x != student, self.team_index[student].students))
+
+    def mark_done(self, student: str):
+        self.marked += self.team_index[student].students
+
+    def skip(self, student: str) -> bool:
+        return student in self.marked
+
+    def remaining(self) -> int:
+        return len(self.students) - len(self.marked)
+
+    def mark_team(self, team: str):
+        self.remaining_teams = list(filter(lambda t: t.name != team, self.remaining_teams))
+
