@@ -21,47 +21,6 @@ KEY_UE_BUILD_STATUS = 'build_status'
 KEY_UE_PRE_EVAL_EX = 'pre_eval_ex_%02d'
 
 
-class TeamIndex:
-
-    def __init__(self, group):
-        self.students = group.student_list()
-        csv_file = group.team_csv()
-        self.index = {}
-        self.marked = []
-        self.team_name = {}
-        with open(csv_file, 'r') as infile:
-            reader = csv.DictReader(infile, teams_csv_fieldnames(), KEY_INVALID, strict=True)
-
-            headers = next(reader, None)
-            if not validate_headers(headers):
-                click.secho('Malformed file: %s. Invalid headers!' % csv_file)
-                exit(1)
-
-            for row in reader:
-                member_1 = row[KEY_TEAM_MEMBER_1]
-                member_2 = row[KEY_TEAM_MEMBER_2]
-                member_3 = row[KEY_TEAM_MEMBER_3]
-
-                self.index[member_1] = [member_2]
-                self.index[member_2] = [member_1]
-                self.team_name[member_1] = row[KEY_TEAM_NAME]
-                self.team_name[member_2] = row[KEY_TEAM_NAME]
-
-                if member_3 is not None and len(member_3) > 0:
-                    self.index[member_3] = [member_1, member_2]
-                    self.index[member_1] += [member_3]
-                    self.index[member_2] += [member_3]
-                    self.team_name[member_3] = row[KEY_TEAM_NAME]
-
-    def other_students(self, student):
-        self.marked += [student]
-        self.marked += self.index[student]
-        return self.index[student]
-
-    def skip(self, student):
-        return student in self.marked
-
-
 def attendance_csv_fieldnames(ep2: Ep2, ue: int) -> [str]:
     if ue == 7:
         return pre_eval_csv_fieldnames_ta()
@@ -125,7 +84,15 @@ def checkout(ctx, group, ue: int):
                 click.secho('Malformed file: %s. Missing column(s)!' % students_file, fg='red')
                 exit(1)
 
-            student_id = row[KEY_STUDENT_ID]
+            student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+            if student_id.is_err():
+                click.secho("[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                            fg='red', bold=True)
+                return
+            else:
+                student_id = student_id.ok()
+
             repo = ep2.ue_repo(student_id)
 
             # clone or pull repo
@@ -150,7 +117,15 @@ def checkout(ctx, group, ue: int):
                 for row in student_list.values():
                     # no validation required, as file is the same
 
-                    student_id = row[KEY_STUDENT_ID]
+                    student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                    if student_id.is_err():
+                        click.secho(
+                            "[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                            fg='red', bold=True)
+                        return
+                    else:
+                        student_id = student_id.ok()
 
                     if curr_ue_type == ExerciseType.Normal:
                         sub_exercises = {KEY_UE_PRE_EVAL_EX % d: '0/0' for d in range(1, ep2.tests_for_exercise(ue) + 1)}
@@ -228,8 +203,6 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
     pre_eval_csv_file = group.pre_eval_csv(ue)  # file for reading
     tmp_file = pre_eval_csv_file + '.tmp'  # file for writing
 
-    print(student)
-
     curr_ue_type = ue_type(ue)
     if curr_ue_type == ExerciseType.Team:
         if points is None:
@@ -251,12 +224,13 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
 
     try:
         team_index = TeamIndex(group)
-    except:
+    except Exception as e:
+        print(e)
         team_index = None
 
-    with open(pre_eval_csv_file, 'r') as infile:
+    with open(pre_eval_csv_file, 'r', encoding='utf-8-sig') as infile:
         f_info.open_write(tmp_file, True)
-        with open(tmp_file, 'w') as outfile:
+        with open(tmp_file, 'w', encoding='utf-8') as outfile:
 
             if curr_ue_type == ExerciseType.Normal:
                 reader = csv.DictReader(infile, fieldnames=pre_eval_csv_fieldnames(ep2, ue), strict=True, restkey=KEY_INVALID)
@@ -294,7 +268,17 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
                     exit(1)
 
                 if curr_ue_type == ExerciseType.Normal:
-                    student_id = row[KEY_STUDENT_ID]
+                    student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                    if student_id.is_err():
+                        click.secho(
+                            "[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                            fg='red', bold=True)
+                        return
+                    else:
+                        student_id = student_id.ok()
+
+                    row[KEY_STUDENT_ID] = student_id
                     if student_id == student:
                         row[KEY_UE_GRADING] = grading
 
@@ -312,7 +296,17 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
                                     fg='red')
                         exit(3)
 
-                    student_id = row[KEY_STUDENT_ID]
+                    student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                    if student_id.is_err():
+                        click.secho(
+                            "[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                            fg='red', bold=True)
+                        return
+                    else:
+                        student_id = student_id.ok()
+                    row[KEY_STUDENT_ID] = student_id
+
                     if student_id == student and not team_index.skip(student_id):
                         row[KEY_UE_TA_POINTS] = points
 
@@ -328,7 +322,7 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
                         team_members = team_index.other_students(student_id)
                         if len(team_members) > 0:
                             click.secho('adding other members of team %s (%s)' % (
-                                team_index.team_name[student_id], team_members))
+                                team_index.team_index[student_id], team_members))
                         for team_member in team_members:
                             row[KEY_STUDENT_ID] = team_member
                             writer.writerow(row)
@@ -345,6 +339,23 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
         'For a list of ungraded exercises run ' + click.style('%s list ungraded' % EVAL_COMMAND_NAME, bold=True) + '.')
 
 
+@cli.command()
+@click.option("--group", required=True, prompt=True, help='name of the group')
+@click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', is_eager=True,
+              type=click.INT)
+@click.option("--idea/--no-idea", default=False,
+              help="load project into the folder, that is monitored by IntelliJ Idea")
+@click.pass_context
+def grade_interactive(ctx, group, ue: int, idea: bool):
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
+    group = ep2.group(group)
+    f_info = FileInformation('Grade')
+
+    students = group.student_info()
+    for student in students:
+        print(student)
+
+
 @cli.command()
 @click.option("--group", required=True, prompt=True, help='name of the group')
 @click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', type=click.INT)
@@ -409,12 +420,21 @@ def submit(ctx, group: str, ue: int):
                     f_info.print_info()
                     exit(1)
 
-                stud_data = students[row[KEY_STUDENT_ID]]
+                student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                if student_id.is_err():
+                    click.secho("[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                                fg='red', bold=True)
+                    return
+                else:
+                    student_id = student_id.ok()
+
+                stud_data = students[student_id]
 
                 if curr_ue_type == ExerciseType.Normal:
                     grading = row[KEY_UE_GRADING]   # for d in range(1, ep2.tests_for_exercise(ue) + 1)]
                     if any(grade == '_' for grade in grading):
-                        click.secho('%s has not been graded yet. aborting!' % row[KEY_STUDENT_ID], fg='red')
+                        click.secho('%s has not been graded yet. aborting!' % student_id, fg='red')
                         f_info.print_info()
                         exit(1)
                         return
@@ -423,7 +443,7 @@ def submit(ctx, group: str, ue: int):
 
                     if not grading_correct:
                         click.secho('the grading for student {} is not correct. (possibly wrong number of subgrades)'
-                                    .format(row[KEY_STUDENT_ID]))
+                                    .format(student_id))
 
                     template.grading = grading
                     if len(template.grading) < len(template.subtasks):
@@ -437,7 +457,7 @@ def submit(ctx, group: str, ue: int):
                 elif curr_ue_type == ExerciseType.Team:
                     points = row[KEY_UE_TA_POINTS]
                     if points == '_':
-                        click.secho('%s has not been graded yet. aborting!' % row[KEY_STUDENT_ID], fg='red')
+                        click.secho('%s has not been graded yet. aborting!' % student_id, fg='red')
                         f_info.print_info()
                         exit(1)
                         return
@@ -451,14 +471,15 @@ def submit(ctx, group: str, ue: int):
 
                 issue = template.__str__()
                 click.echo(issue.replace('\n\n', '\n'))
-                click.echo('Issue for %s - %s, %s\n' % (row[KEY_STUDENT_ID], stud_data[KEY_STUDENT_NAME_LAST].upper(),
+                click.echo('Issue for %s - %s, %s\n' % (student_id, stud_data[KEY_STUDENT_NAME_LAST].upper(),
                                                         stud_data[KEY_STUDENT_NAME_FIRST]))
 
                 click.echo('\n' + ('=' * 30) + '\n')
-                issues += [(row[KEY_STUDENT_ID], issue)]
+                issues += [(student_id, issue)]
 
                 row[KEY_UE_FEEDBACK] = escape_csv_string(row[KEY_UE_FEEDBACK])
                 row[KEY_UE_REMARKS] = escape_csv_string(row[KEY_UE_REMARKS])
+                row[KEY_STUDENT_ID] = student_id
 
                 writer.writerow(row)
 
