@@ -201,14 +201,7 @@ def submission(ctx, group, ue, late, idea, grade, single):
         click.confirm('You have chosen exercise type %s even tough ad hoc exercise %s is marked as %s. Continue?' %
                       ('Single' if single else 'Team', ue, curr_adhoc_type.name), abort=True)
 
-    index = StudentIndex(group, ue, single)
-
-    single |= index.single
-
-    if len(index.remaining) == 0:
-        click.echo('No students remaining. Call ' + click.style('adhoc grade', bold=True) +
-                   ' to change the grading of a student')
-        return
+    index = TeamIndex(group)
 
     csv_file = group.exercise_test_csv(ue)
     csv_path = csv_file[:csv_file.rindex(os.sep)]  # needed to create parent dir
@@ -218,108 +211,204 @@ def submission(ctx, group, ue, late, idea, grade, single):
     except OSError:  # directory already exists
         pass
 
-    append = os.path.exists(csv_file)  # if file already exists, don't rewrite header
+    file_exists = os.path.exists(csv_file)
 
     f_info.open_write(csv_file, True)
 
-    with open(csv_file, 'a') as csvfile:
-        writer = csv.DictWriter(csvfile, fieldnames=ex_test_csv_fieldnames(), lineterminator='\n')
+    if not single:
+        if file_exists:
+            with open(csv_file, 'r') as csvfile:
+                reader = csv.DictReader(csvfile, ex_test_csv_fieldnames(), KEY_INVALID, strict=True)
 
-        if not append:  # write header if file is new
-            writer.writeheader()
-
-        click.echo("Please enter submission information according to the prompts. Empty input to quit")
-
-        # loop over input
-        while len(index.remaining) > 0:
-
-            # prompt owner of working repository
-            repo_owner = prompt_manual("Mat.No. Repo", suffix=':   ', default="", show_default=False,
-                                       type=click.Choice(index.auto_fill_index() + [""]), exit_condition=[''])
-
-            if repo_owner.is_err():
-                break
-
-            repo_owner = repo_owner.ok()
-
-            index.mark_done(repo_owner)
-            if not single:
-                auto_fill = index.auto_fill_team(repo_owner)
-
-                if len(index.remaining) == 0:
-                    click.secho('Invalid state: no student ids remaining!')
+                headers = next(reader, None)
+                if not validate_headers(headers):
+                    click.secho('Malformed file: %s. Invalid headers!' % csv_path)
                     exit(1)
 
-                if len(auto_fill) == 1:
-                    editor = prompt_manual("Mat.No. Writer", type=click.Choice(index.auto_fill_index()), default=auto_fill[0],
-                                           show_default=True, suffix=': ')
-                else:
-                    editor = prompt_manual("Mat.No. Writer", type=click.Choice(index.auto_fill_index()), show_default=True,
-                                           suffix=': ')
+                old_rows = []
 
-                if editor.is_err():
+                for row in reader:
+                    student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                    if student_id.is_err():
+                        click.secho(
+                            "[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                            fg='red', bold=True)
+                        return
+                    else:
+                        student_id = student_id.ok()
+
+                    row[KEY_STUDENT_ID] = student_id
+                    old_rows += [row]
+
+                    team = row[KEY_EX_TEST_TEAM]
+                    index.mark_team(team)
+
+        with open(csv_file, 'w') as csvfile:
+            writer = csv.DictWriter(csvfile, fieldnames=ex_test_csv_fieldnames(), lineterminator='\n')
+
+            writer.writeheader()
+
+            if file_exists:
+                for row in old_rows:
+                    writer.writerow(row)
+
+            if len(index.remaining_teams) == 0:
+                click.echo('No teams left to add, use ' + click.style('ep2_ex_text man_submission', bold=True)
+                           + ' to add a submission manually')
+
+            for team in index.remaining_teams:
+                click.secho('Enter repository owner for team {}'.format(team.name))
+
+                for i, member in enumerate(team.students):
+                    click.secho('{}: {}'.format(i, member))
+
+                repo_owner = prompt_manual("Repo Owner", suffix=': ', show_default=False, show_choices=True,
+                                           type=click.Choice([str(i) for i in range(0, len(team.students))] + ['s']))
+
+                if repo_owner.is_err():
                     break
 
-                editor = editor.ok()
+                if repo_owner.ok() == 's':
+                    click.secho('Skipping team {}'.format(team.name), color='yellow')
+                    continue
 
-                index.mark_done(editor)
-                if editor in auto_fill:
-                    auto_fill.remove(editor)
-                else:
-                    auto_fill = []
-                # allow for teams of 3 people
+                repo_owner = team.students[int(repo_owner.ok())]
 
-                if len(auto_fill) == 1:
-                    third = prompt_manual("Mat.No. Third", suffix=':  ', type=click.Choice(index.auto_fill_index() + ['none']),
-                                          default=auto_fill[0], show_default=True, exit_condition=['', 'none'])
-                else:
-                    third = prompt_manual("Mat.No. Third", suffix=':  ', type=click.Choice(index.auto_fill_index() + ['none']),
-                                          default='none', show_default=False, exit_condition=['', 'none'])
+                index.mark_done(repo_owner)
 
-                if third.is_err():
+                if len(team.students) == 3:
+                    other_students = index.other_students(repo_owner)
+                    click.secho('Enter writer for team {}'.format(team.name))
+
+                    for i, member in enumerate(other_students):
+                        click.secho('{}: {}'.format(i, member))
+
+                    editor = prompt_manual("Mat.No. Writer", type=click.Choice(str(i) for i in range(0, len(other_students))),
+                                           show_default=False, suffix=': ')
+                    if editor.is_err():
+                        break
+
+                    third = other_students[(int(editor.ok()) + 1) % 2]
+                    editor = other_students[int(editor.ok())]
+                elif len(team.students) == 2:
+                    editor = index.other_students(repo_owner)[0]
                     third = None
-                else:
-                    third = third.ok()
 
                 if third is not None and third != 'none':
                     members = [(repo_owner, 'o'), (editor, 'e'), (third, 't')]
                     index.mark_done(third)
                 else:
                     members = [(repo_owner, 'o'), (editor, 'e')]
-            else:
-                members = [(repo_owner, 'o')]
 
-            team = team_name(map(lambda x: x[0], members))
+                if idea:
+                    current_uebung = ep2.ue_repo(repo_owner)
 
-            if idea:
-
-                git_path = ep2.config.get("Local", "GitHome")
-                uebung_path = os.path.join(git_path, "uebung")
-
-                current_uebung = os.path.join(uebung_path, repo_owner)
-                if not ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST_LATE % ue):
-                    if not ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST % ue):
-                        click.secho("Could not checkout submission from " + repo_owner +
-                                    ", no tagged commit found", fg='red')
+                    if not ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST_LATE % ue):
+                        if not ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST % ue):
+                            click.secho("Could not checkout submission from " + repo_owner +
+                                        ", no tagged commit found", fg='red')
+                        else:
+                            ep2.idea_checkout(repo_owner)
+                            click.secho('Project %s loaded into IntelliJ Idea' % repo_owner, fg='green')
                     else:
                         ep2.idea_checkout(repo_owner)
                         click.secho('Project %s loaded into IntelliJ Idea' % repo_owner, fg='green')
+
+                if grade:
+                    points = click.prompt("Points", type=click.IntRange(0, 4), show_choices=True)
+                    remarks = click.prompt("Remarks", default='', show_default=True)
+                    feedback = click.prompt("Feedback", default='', show_default=True)
                 else:
-                    ep2.idea_checkout(repo_owner)
-                    click.secho('Project %s loaded into IntelliJ Idea' % repo_owner, fg='green')
+                    points = '_'
+                    remarks = ''
+                    feedback = ''
 
-            if grade:
-                points = click.prompt("Points", type=click.IntRange(0, 4), show_choices=True)
-            else:
-                points = '_'
+                for member in members:
+                    writer.writerow(
+                        {KEY_STUDENT_ID: member[0], KEY_EX_TEST_ROLE: member[1], KEY_EX_TEST_POINTS: points,
+                         KEY_EX_TEST_LATE: 1 if late else 0, KEY_EX_TEST_TEAM: team.name,
+                         KEY_EX_TEST_REMARKS: remarks, KEY_EX_TEST_FEEDBACK: feedback})
 
-            for member in members:
-                writer.writerow(
-                    {KEY_STUDENT_ID: member[0], KEY_EX_TEST_ROLE: member[1], KEY_EX_TEST_POINTS: points,
-                     KEY_EX_TEST_LATE: 1 if late else 0, KEY_EX_TEST_TEAM: team,
-                     KEY_EX_TEST_REMARKS: '', KEY_EX_TEST_FEEDBACK: ''})
+                click.secho('submission ok', fg='green')
 
-            click.secho('submission ok', fg='green')
+    else:
+        with open(csv_file, 'w') as csvfile:
+            writer = csv.DictWriter(csvfile, fieldnames=ex_test_csv_fieldnames(), lineterminator='\n')
+
+            writer.writeheader()
+
+            action = prompt_manual('Create submissions for: (all, manual, check)', show_choices=True,
+                                   show_default=True, default='a', type=click.Choice(['a', 'c']), suffix='> ')
+
+            if action.is_ok():
+                action = action.ok()
+
+                if action == 'a':
+                    for student in index.students:
+                        if idea:
+                            current_uebung = ep2.ue_repo(student)
+
+                            if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                                       TAG_NAME_EX_TEST_LATE % ue):
+                                if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                                           TAG_NAME_EX_TEST % ue):
+                                    click.secho("Could not checkout submission from " + student +
+                                                ", no tagged commit found", fg='red')
+                                else:
+                                    ep2.idea_checkout(student)
+                                    click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+                            else:
+                                ep2.idea_checkout(student)
+                                click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+
+                        if grade:
+                            points = click.prompt("Points for {}".format(student), type=click.IntRange(0, 4),
+                                                  show_choices=True)
+                            remarks = click.prompt("Remarks", default='', show_default=True)
+                            feedback = click.prompt("Feedback", default='', show_default=True)
+                        else:
+                            points = '_'
+                            remarks = ''
+                            feedback = ''
+
+                        writer.writerow(
+                            {KEY_STUDENT_ID: student, KEY_EX_TEST_ROLE: 'o', KEY_EX_TEST_POINTS: points,
+                             KEY_EX_TEST_LATE: 1 if late else 0, KEY_EX_TEST_TEAM: student,
+                             KEY_EX_TEST_REMARKS: remarks, KEY_EX_TEST_FEEDBACK: feedback})
+                elif action == 'c':
+
+                    for student in sorted(index.students):
+                        if click.confirm('Add submission for student {}?'.format(student)):
+                            if idea:
+                                current_uebung = ep2.ue_repo(student)
+
+                                if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                                           TAG_NAME_EX_TEST_LATE % ue):
+                                    if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                                               TAG_NAME_EX_TEST % ue):
+                                        click.secho("Could not checkout submission from " + student +
+                                                    ", no tagged commit found", fg='red')
+                                    else:
+                                        ep2.idea_checkout(student)
+                                        click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+                                else:
+                                    ep2.idea_checkout(student)
+                                    click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+
+                            if grade:
+                                points = click.prompt("Points", type=click.IntRange(0, 4), show_choices=True)
+                                remarks = click.prompt("Remarks", default='', show_default=True)
+                                feedback = click.prompt("Feedback", default='', show_default=True)
+                            else:
+                                points = '_'
+                                remarks = ''
+                                feedback = ''
+
+                            writer.writerow(
+                                {KEY_STUDENT_ID: student, KEY_EX_TEST_ROLE: 'o', KEY_EX_TEST_POINTS: points,
+                                 KEY_EX_TEST_LATE: 1 if late else 0, KEY_EX_TEST_TEAM: student,
+                                 KEY_EX_TEST_REMARKS: remarks, KEY_EX_TEST_FEEDBACK: feedback})
 
     f_info.print_info()
     if not idea:
@@ -328,29 +417,106 @@ def submission(ctx, group, ue, late, idea, grade, single):
         click.echo('To grade the exercises run ' + click.style('%s grade' % EX_TEST_COMMAND_NAME, bold=True) + '.')
         click.echo('To load the projects into IntelliJ IDEA run ' + click.style('ep2_util idea', bold=True) + '.')
 
-    click.echo('Submit your results using ' + click.style('adhoc submit', bold=True) + '.')
+    click.echo('Submit your results using ' + click.style('ep2_ex_test submit', bold=True) + '.')
+
+
+@cli.command()
+@click.option("--group", required=True, prompt=True, help='name of the group')
+@click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', type=int)
+@click.option("--student", required=True, prompt=True, help='student id of the student, that should be graded',
+              callback=validate_student_id)
+@click.option("--role", prompt=True, type=click.Choice(['o', 'w', 't']), help='the role of the student for this exercise test')
+@click.option("--late/--on-time", default=False, help='if submission was late')
+@click.option("--team", prompt=True, help='the name of the team to add (this will not be verified!)')
+@click.pass_context
+def man_submission(ctx, group: str, ue: int, student: str, late: bool, role: str, team: str):
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
+    group = ep2.group(group)
+    f_info = FileInformation('Submission')
+
+    csv_file = group.exercise_test_csv(ue)
+    csv_path = csv_file[:csv_file.rindex(os.sep)]  # needed to create parent dir
+
+    try:
+        os.makedirs(csv_path)
+    except OSError:  # directory already exists
+        pass
+
+    file_exists = os.path.exists(csv_file)
+
+    f_info.open_write(csv_file, True)
+    team_points = None
+    team_feedback = None
+    team_remarks = None
+
+    if file_exists:
+        with open(csv_file, 'r') as csvfile:
+            reader = csv.DictReader(csvfile, ex_test_csv_fieldnames(), KEY_INVALID, strict=True)
+
+            headers = next(reader, None)
+            if not validate_headers(headers):
+                click.secho('Malformed file: %s. Invalid headers!' % csv_path)
+                exit(1)
+
+            old_rows = []
+
+            for row in reader:
+                student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                if student_id.is_err():
+                    click.secho(
+                        "[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                        fg='red', bold=True)
+                    return
+                else:
+                    student_id = student_id.ok()
+
+                if student_id == student:
+                    click.secho('Student already in file, overwriting record!', fg='yellow')
+                else:
+                    row[KEY_STUDENT_ID] = student_id
+                    old_rows += [row]
+
+                if team == row[KEY_EX_TEST_TEAM]:
+                    if row[KEY_EX_TEST_POINTS] != '_':
+                        team_points = row[KEY_EX_TEST_POINTS]
+                        team_feedback = row[KEY_EX_TEST_FEEDBACK]
+                        team_remarks = row[KEY_EX_TEST_REMARKS]
+
+    with open(csv_file, 'w') as csvfile:
+        writer = csv.DictWriter(csvfile, fieldnames=ex_test_csv_fieldnames(), lineterminator='\n')
+
+        writer.writeheader()
+
+        if file_exists:
+            for row in old_rows:
+                writer.writerow(row)
+
+        writer.writerow(
+            {KEY_STUDENT_ID: student, KEY_EX_TEST_ROLE: role,
+             KEY_EX_TEST_POINTS: team_points if team_points is not None else '_',
+             KEY_EX_TEST_LATE: 1 if late else 0, KEY_EX_TEST_TEAM: team,
+             KEY_EX_TEST_REMARKS: team_remarks if team_remarks is not None else '',
+             KEY_EX_TEST_FEEDBACK: team_feedback if team_feedback is not None else ''})
+
+        click.secho('submission ok', fg='green')
+
+    f_info.print_info()
+
+    click.echo('Submit your results using ' + click.style('ep2_ex_test submit', bold=True) + '.')
 
 
 def prompt_manual(prompt:str, suffix:str, type:click.Choice, show_default: bool, default:str = None,
-                  exit_condition: [str] = []) -> Result[str, str]:
+                  exit_condition=None, show_choices=False) -> Result[str, str]:
+    if exit_condition is None:
+        exit_condition = []
     prompt_result = click.prompt(prompt, prompt_suffix=suffix, show_default=show_default,
-                         type=type, show_choices=False, default=default)
+                         type=type, show_choices=show_choices, default=default)
 
     if prompt_result in exit_condition:
         return Err('exit condition reached')
 
-    verification_result = verify_and_normalize_student_id(prompt_result)
-
-    while verification_result.is_err():
-        prompt_result = click.prompt(prompt, prompt_suffix=suffix, show_default=show_default,
-                                     type=type, show_choices=False, default=default)
-
-        if prompt_result in exit_condition:
-            return Err('exit condition reached')
-
-        verification_result = verify_and_normalize_student_id(prompt_result)
-
-    return verification_result
+    return Ok(prompt_result)
 
 
 @cli.command()
@@ -398,10 +564,19 @@ def checkout(ctx, group, ue: int):
                 exit(1)
 
             if row[KEY_EX_TEST_ROLE] == 'o':
-                current_uebung = os.path.join(uebung_path, row[KEY_STUDENT_ID])
-                if not ep2.clone_or_update(current_uebung, ep2.ue_repo(row[KEY_STUDENT_ID]), TAG_NAME_EX_TEST_LATE % ue):
-                    if not ep2.clone_or_update(current_uebung, ep2.ue_repo(row[KEY_STUDENT_ID]), TAG_NAME_EX_TEST % ue):
-                        click.secho("Could not checkout submission from " + row[KEY_STUDENT_ID] +
+                student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                if student_id.is_err():
+                    click.secho("[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                                fg='red', bold=True)
+                    return
+                else:
+                    student_id = student_id.ok()
+
+                current_uebung = os.path.join(uebung_path, student_id)
+                if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST_LATE % ue):
+                    if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST % ue):
+                        click.secho("Could not checkout submission from " + student_id +
                                     ", no tagged commit found", fg='red')
 
     click.echo('To grade the exercises run ' + click.style('%s grade' % EX_TEST_COMMAND_NAME, bold=True) + '.')
@@ -409,16 +584,15 @@ def checkout(ctx, group, ue: int):
 
 
 @cli.command()
-@click.option("--repo-owner", required=True, prompt=True, help='the matriculation number of the person, in whose '
-                                                               'repository the exercise was written in',
-              callback=validate_student_id)
+@click.option("--team", required=True, prompt=True, help='the name of the team or the student id of the student that '
+                                                         'should be graded')
 @click.option("--points", required=True, prompt=True, help='points graded for this exercise', type=click.IntRange(0, 4))
 @click.option("--group", required=True, prompt=True, help='name of the group')
 @click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero')
 @click.option("--remarks", prompt=True, default='', help='optional remarks for the lecturer')
 @click.option("--feedback", prompt=True, default='', help='feedback for the students')
 @click.pass_context
-def grade(ctx, repo_owner, points, ue, group, remarks, feedback):
+def grade(ctx, team, points, ue, group, remarks, feedback):
     """This command grades a single submission by updating the submissions CSV entry and creating an
     issue for the project in which the ad-hoc exercise was written in."""
     f_info = FileInformation('Grade')
@@ -428,24 +602,6 @@ def grade(ctx, repo_owner, points, ue, group, remarks, feedback):
     csv_file = group.exercise_test_csv(ue)
     tmp_file = csv_file + ".tmp"  # tmp file for editing
 
-    team = None
-
-    with open(csv_file, 'r') as infile:
-        reader = csv.DictReader(infile, ex_test_csv_fieldnames(), KEY_INVALID, strict=True)
-
-        headers = next(reader, None)
-        if not validate_headers(headers):
-            click.secho('Malformed file: %s. Invalid headers!' % csv_file)
-            exit(1)
-
-        for row in reader:
-            if KEY_INVALID in row:
-                click.secho('Malformed file: %s' % csv_file, fg='red')
-                exit(1)
-
-            if row[KEY_STUDENT_ID] == repo_owner:
-                team = row[KEY_EX_TEST_TEAM]
-
     f_info.open_write(tmp_file, True)
 
     with open(csv_file, 'r') as infile:
@@ -473,6 +629,16 @@ def grade(ctx, repo_owner, points, ue, group, remarks, feedback):
                     f_info.print_info()
                     exit(1)
 
+                student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                if student_id.is_err():
+                    click.secho("[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                                fg='red', bold=True)
+                    return
+                else:
+                    student_id = student_id.ok()
+
+                row[KEY_STUDENT_ID] = student_id
                 if row[KEY_EX_TEST_TEAM] == team:  # if team matches, update points
                     row[KEY_EX_TEST_POINTS] = points
                     if remarks is not None and remarks != '':
@@ -514,7 +680,7 @@ def submit(ctx, group, ue):
     students = group.student_list()
     student_details = group.student_info()
 
-    issues = []  # type: [(str, unicode)]
+    issues = []
 
     template = Template(file=ep2.template_path('adhoc.tmpl'))
 
@@ -546,28 +712,39 @@ def submit(ctx, group, ue):
                     f_info.print_info()
                     exit(1)
 
+                student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                if student_id.is_err():
+                    click.secho("[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                                fg='red', bold=True)
+                    return
+                else:
+                    student_id = student_id.ok()
+
+                row[KEY_STUDENT_ID] = student_id
+
                 if row[KEY_EX_TEST_POINTS] == '_':
-                    click.secho('%s has not been graded yet. aborting!' % row[KEY_STUDENT_ID], fg='red')
+                    click.secho('%s has not been graded yet. aborting!' % student_id, fg='red')
                     f_info.print_info()
                     exit(1)
 
                 row[KEY_EX_TEST_FEEDBACK] = escape_csv_string(row[KEY_EX_TEST_FEEDBACK])
                 row[KEY_EX_TEST_REMARKS] = escape_csv_string(row[KEY_EX_TEST_REMARKS])
 
-                details = student_details[row[KEY_STUDENT_ID]]
+                details = student_details[student_id]
 
                 template.points = row[KEY_EX_TEST_POINTS]
                 template.student_feedback = row[KEY_EX_TEST_FEEDBACK]
                 template.student_gender = details[KEY_STUDENT_GENDER]
                 template.attended = True
 
-                students.remove(row[KEY_STUDENT_ID])
+                students.remove(student_id)
 
                 issue = template.__str__()
-                click.echo('Issue for student %s' % row[KEY_STUDENT_ID], nl=True)
+                click.echo('Issue for student %s' % student_id, nl=True)
                 click.echo(issue.replace('\n\n', '\n'), nl=True)
                 click.echo(('=' * 30), nl=True)
-                issues += [(row[KEY_STUDENT_ID], issue)]
+                issues += [(student_id, issue)]
 
                 writer.writerow(row)
 
@@ -632,7 +809,7 @@ def list_ungraded(ctx, ue, group):
     group = ep2.group(group)
 
     csv_file = group.exercise_test_csv(ue)
-    empty = True
+    ungraded_teams: [str] = []
 
     try:
         with open(csv_file, 'r') as infile:
@@ -652,15 +829,28 @@ def list_ungraded(ctx, ue, group):
                     click.secho('Malformed file: %s. Missing column(s)!' % csv_file, fg='red')
                     exit(1)
 
-                if row[KEY_EX_TEST_POINTS] == '_' and row[KEY_EX_TEST_ROLE] == 'o':
-                    click.echo(row[KEY_STUDENT_ID])
-                    empty = False
+                student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                if student_id.is_err():
+                    click.secho("[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                                fg='red', bold=True)
+                    return
+
+                if row[KEY_EX_TEST_POINTS] == '_':
+                    ungraded_teams += [row[KEY_EX_TEST_TEAM]]
     except IOError:
         click.secho('no submission for ' + group.name + ' ue ' + ue, fg='red')
         exit(1)
 
-    if empty:
-        click.secho('empty', fg='green')
+    if len(ungraded_teams) == 0:
+        click.secho('all graded!', fg='green', bold=True)
+    else:
+        click.secho('Ungraded teams/students:', fg='yellow', bold='True')
+        for team in ungraded_teams:
+            import sys
+            click.echo("    - {} (call: {})".format(team, click.style('{} --group {} --ue {} --team {}'
+                                                                      .format(sys.argv[0], group.name, ue, team),
+                                                                      bold=True)))
 
 
 if __name__ == '__main__':
