@@ -150,7 +150,7 @@ def util_tag(ctx, group, tag, teams):
 def idea(ctx, project):
     """Copies a project to the a directory, that is monitored by IDEA, so it can be opened quickly"""
     ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
-    ep2.idea_checkout(project)
+    ep2.idea_checkout(project, 'do13', 1, True, True)
 
 
 if __name__ == '__main__':
